# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Oekolabel(StrEnum):
    """
    Aufzählung der Labels für Öko-Strom von verschiedenen Herausgebern.
    """

    ENERGREEN = "ENERGREEN"  #: ENERGREEN
    GASGREEN_GRUENER_STROM = "GASGREEN_GRUENER_STROM"  #: GASGREEN_GRUENER_STROM
    GASGREEN = "GASGREEN"  #: GASGREEN
    GRUENER_STROM_GOLD = "GRUENER_STROM_GOLD"  #: GRUENER_STROM_GOLD
    GRUENER_STROM_SILBER = "GRUENER_STROM_SILBER"  #: GRUENER_STROM_SILBER
    GRUENER_STROM = "GRUENER_STROM"  #: GRUENER_STROM
    GRUENES_GAS = "GRUENES_GAS"  #: GRUENES_GAS
    NATURWATT_STROM = "NATURWATT_STROM"  #: NATURWATT_STROM
    OK_POWER = "OK_POWER"  #: OK_POWER
    RENEWABLE_PLUS = "RENEWABLE_PLUS"  #: RENEWABLE_PLUS
    WATERGREEN = "WATERGREEN"  #: WATERGREEN
    WATERGREEN_PLUS = "WATERGREEN_PLUS"  #: WATERGREEN_PLUS



"""
Contains Zaehlwerk class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

from ..com.konzessionsabgabe import Konzessionsabgabe
from ..com.verwendungszweckpromarktrolle import VerwendungszweckProMarktrolle
from ..com.zaehlzeitregister import Zaehlzeitregister
from ..enum.energierichtung import Energierichtung
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.verbrauchsart import Verbrauchsart
from ..enum.waermenutzung import Waermenutzung
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=no-name-in-module
# pylint: disable=no-name-in-module

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Zaehlwerk(COM):
    """
    Mit dieser Komponente werden Zählwerke modelliert.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zaehlwerk.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehlwerk JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zaehlwerk.json>`_

    """

    zaehlwerk_id: Optional[str] = None  # Identifikation des Zählwerks (Registers) innerhalb des Zählers.
    # Oftmals eine laufende Nummer hinter der Zählernummer. Z.B. 47110815_1
    bezeichnung: Optional[str] = None  # Zusätzliche Bezeichnung, z.B. Zählwerk_Wirkarbeit.
    richtung: Optional[Energierichtung] = None  # Die Energierichtung, Einspeisung oder Ausspeisung.
    obis_kennzahl: Optional[
        str
    ] = None  # Die OBIS-Kennzahl für das Zählwerk, die festlegt, welche auf die gemessene Größe mit dem Stand gemeldet wird.
    # Nur Zählwerkstände mit dieser OBIS-Kennzahl werden an diesem Zählwerk registriert.
    wandlerfaktor: Optional[
        Decimal
    ] = None  # Mit diesem Faktor wird eine Zählerstandsdifferenz multipliziert, um zum eigentlichen Verbrauch im Zeitraum
    # zu kommen.
    einheit: Optional[Mengeneinheit] = None  # Die Einheit der gemessenen Größe, z.B. kWh
    ist_schwachlastfaehig: Optional[bool] = None  #: Schwachlastfaehigkeit
    verwendungszwecke: Optional[
        list[VerwendungszweckProMarktrolle]
    ] = None  #: Verwendungungszweck der Werte Marktlokation
    verbrauchsart: Optional[Verbrauchsart] = None  #: Stromverbrauchsart/Verbrauchsart Marktlokation
    ist_unterbrechbar: Optional[bool] = None  #: Unterbrechbarkeit Marktlokation
    waermenutzung: Optional[Waermenutzung] = None  #: Wärmenutzung Marktlokation
    konzessionsabgabe: Optional[Konzessionsabgabe] = None  #: Konzessionsabgabe
    ist_steuerbefreit: Optional[bool] = None  #: Steuerbefreiung
    vorkommastelle: Optional[int] = None  #: Anzahl der Vorkommastellen
    nachkommastelle: Optional[int] = None  #: Anzahl der Nachkommastellen
    ist_abrechnungsrelevant: Optional[bool] = None  #: Abrechnungsrelevant
    anzahlAblesungen: Optional[int] = None  #: Anzahl Ablesungen pro Jahr
    zaehlzeitregister: Optional[
        Zaehlzeitregister
    ] = None  #: Erweiterte Definition der Zählzeit in Bezug auf ein Register



import pytest

from bo4e import Marktteilnehmer, Preisblatt, Preisposition, Preisstatus, Sparte, Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestPreisblatt:
    @pytest.mark.parametrize(
        "preisblatt",
        [
            pytest.param(
                Preisblatt(
                    bezeichnung="foo",
                    sparte=Sparte.STROM,
                    preisstatus=Preisstatus.ENDGUELTIG,
                    preispositionen=[Preisposition()],
                    gueltigkeit=Zeitraum(),
                    herausgeber=Marktteilnehmer(),
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, preisblatt: Preisblatt) -> None:
        """
        Test de-/serialisation of Preisblatt.
        """
        assert_serialization_roundtrip(preisblatt)



"""
Version information for the bo4e package.
"""

import re
from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("bo4e")
except PackageNotFoundError:
    __version__ = "0.0.0"
# Please keep this name in sync with the name of the project in pyproject.toml
# This name is the name of the package on pypi.org
if re.match(r"^(\d+\.\d+\.\d+)(rc\d+)?$", __version__):

    def _repl(match: re.Match[str]) -> str:
        if match.group(2) is not None:
            return f"v{match.group(1)}-{match.group(2)}"
        return f"v{match.group(1)}"

    __gh_version__ = re.sub(r"^(\d+\.\d+\.\d+)(rc\d+)?$", _repl, __version__)
else:
    __gh_version__ = f"v{__version__}"



import pytest
from pydantic import ValidationError

from bo4e import Geschaeftsobjekt, Typ
from bo4e.zusatzattribut import ZusatzAttribut
from tests.serialization_helper import assert_serialization_roundtrip


class TestGeschaeftsobjekt:
    @pytest.mark.parametrize(
        "geschaeftsobjekt",
        [
            Geschaeftsobjekt(
                typ=Typ.ENERGIEMENGE,
                version="2",
                zusatz_attribute=[
                    ZusatzAttribut(name="HOCHFREQUENZ_HFSAP_100", wert="12345"),
                    ZusatzAttribut(name="Schufa-ID", wert="aksdlakoeuhn"),
                ],
            ),
            Geschaeftsobjekt(
                typ=Typ.ENERGIEMENGE,
                version="2",
                zusatz_attribute=[],
            ),
        ],
    )
    def test_serialization_roundtrip(self, geschaeftsobjekt: Geschaeftsobjekt) -> None:
        """
        Test de-/serialisation of Geschaeftsobjekt
        """
        assert_serialization_roundtrip(geschaeftsobjekt)

    def test_no_list_in_externen_referenzen(self) -> None:
        with pytest.raises(ValidationError) as excinfo:
            _ = Geschaeftsobjekt(
                typ=Typ.ENERGIEMENGE,
                zusatz_attribute=ZusatzAttribut(name="Schufa-ID", wert="aksdlakoeuhn"),  # type: ignore[arg-type]
            )
        # The error message is completely broken, but who cares
        assert "2 validation error" in str(excinfo.value)
        assert "type=model_type" in str(excinfo.value)



from decimal import Decimal

import pytest

from bo4e import Mengeneinheit, Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestZeitraum:
    @pytest.mark.parametrize(
        "zeitraum",
        [
            pytest.param(
                Zeitraum(
                    einheit=Mengeneinheit.TAG,
                    dauer=Decimal(21),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, zeitraum: Zeitraum) -> None:
        """
        Test de-/serialisation of Zeitraum.
        """
        assert_serialization_roundtrip(zeitraum)



import pytest

from bo4e import Geraet, Geraeteklasse, Geraetetyp
from tests.serialization_helper import assert_serialization_roundtrip

example_geraet = Geraet(
    geraetenummer="0815",
    geraeteklasse=Geraeteklasse.WANDLER,
    geraetetyp=Geraetetyp.BLOCKSTROMWANDLER,
    bezeichnung="56k Modem",
)


class TestGeraet:
    @pytest.mark.parametrize(
        "geraet",
        [
            pytest.param(
                Geraet(
                    geraetenummer="0815",
                    bezeichnung="56k Modem",
                    geraeteklasse=Geraeteklasse.WANDLER,
                    geraetetyp=Geraetetyp.BLOCKSTROMWANDLER,
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, geraet: Geraet) -> None:
        """
        Test de-/serialisation of Geraet
        """
        assert_serialization_roundtrip(geraet)



"""
Contains Region class and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

from pydantic import Field

from ..com.regionskriterium import Regionskriterium
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Region(Geschaeftsobjekt):
    """
    Modellierung einer Region als Menge von Kriterien, die eine Region beschreiben

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Region.svg" type="image/svg+xml"></object>

    .. HINT::
        `Region JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Region.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.REGION
    #: Bezeichnung der Region
    bezeichnung: Optional[str] = None

    #: Positivliste der Kriterien zur Definition der Region
    positiv_liste: Optional[list[Regionskriterium]] = None

    #: Negativliste der Kriterien zur Definition der Region
    negativ_liste: Optional[list[Regionskriterium]] = None



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Bemessungsgroesse(StrEnum):
    """
    Zur Abbildung von Messgrössen und zur Verwendung in energiewirtschaftlichen Berechnungen.
    """

    WIRKARBEIT_EL = "WIRKARBEIT_EL"  #: elektrische Wirkarbeit
    LEISTUNG_EL = "LEISTUNG_EL"  #: elektrische Leistung
    BLINDARBEIT_KAP = "BLINDARBEIT_KAP"  #: Blindarbeit kapazitiv
    BLINDARBEIT_IND = "BLINDARBEIT_IND"  #: Blindarbeit induktiv
    BLINDLEISTUNG_KAP = "BLINDLEISTUNG_KAP"  #: Blindleistung kapazitiv
    BLINDLEISTUNG_IND = "BLINDLEISTUNG_IND"  #: Blindleistung induktiv
    WIRKARBEIT_TH = "WIRKARBEIT_TH"  #: thermische Wirkarbeit
    LEISTUNG_TH = "LEISTUNG_TH"  #: thermische Leistung
    VOLUMEN = "VOLUMEN"  #: Volumen
    VOLUMENSTROM = "VOLUMENSTROM"  #: Volumenstrom (Volumen/Zeit)
    BENUTZUNGSDAUER = "BENUTZUNGSDAUER"  #: Benutzungsdauer (Arbeit/Leistung)
    ANZAHL = "ANZAHL"  #: Darstellung einer Stückzahl



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Organisationstyp(StrEnum):
    """
    Hier wird festgelegt, ob der Geschäftspartner eine Person, eine Firma oder etwas anderes ist.
    """

    PRIVATPERSON = "PRIVATPERSON"  #: B2C
    UNTERNEHMEN = "UNTERNEHMEN"  #: B2B
    KOMMUNALE_EINRICHTUNG = "KOMMUNALE_EINRICHTUNG"  #: B2A
    STAATLICHE_BEHOERDE = "STAATLICHE_BEHOERDE"  #: B2G
